@@ -7,7 +7,7 @@ use strict;
 use warnings;
 
 BEGIN {
-  $REST::Neo4p::ParseStream::VERSION = '0.4000';
+  $REST::Neo4p::ParseStream::VERSION = '0.3000';
 }
 
 our @EXPORT = qw/j_parse/;# j_parse_object j_parse_array /;
@@ -70,14 +70,10 @@ sub j_parse_array {
     };
     if (defined $elt) {
       $last_text = $j->incr_text;
-      if ($j->incr_text =~ m/^\]}\],/) { # JSON::XS <=3.04 transaction kludge 
+      if ($j->incr_text =~ m/^\]}\],/) { # transaction kludge)
 	$j->incr_text =~ s/(\])(}\],)//;
 	$done=1;
       }
-      elsif ($elt eq 'transaction') { # JSON::XS 4.02 transaction kludge
-	$j->incr_text = '"transaction"'.$j->incr_text;
-	return node(undef,undef);
-      }
       else {
 	$j->incr_text =~ s/^(\s*,\s*)|(\s*\]\s*)//;
 	$done = !!$2;
@@ -253,7 +249,6 @@ REST::Neo4p::ParseStream - Parse Neo4j REST responses on the fly
 =head1 SYNOPSIS
 
  Not for human consumption.
- This module is ignored by the Neo4j::Driver-based agent.
 
 =head1 DESCRIPTION
 
@@ -296,7 +291,7 @@ L<HOP::Stream>, L<JSON::XS/"INCREMENTAL PARSING">.
 
 =head1 LICENSE
 
-Copyright (c) 2012-2020 Mark A. Jensen. This program is free software; you
+Copyright (c) 2012-2014 Mark A. Jensen. This program is free software; you
 can redistribute it and/or modify it under the same terms as Perl
 itself.
 
